/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dawn.h"

typedef uint8_t inputbind_t;
typedef bool_t inputstate_t;

#define INPUT_BIND_UP 0x00
#define INPUT_BIND_DOWN 0x01
#define INPUT_BIND_LEFT 0x02
#define INPUT_BIND_RIGHT 0x03
#define INPUT_BIND_ACCEPT 0x04
#define INPUT_BIND_CANCEL 0x05
#define INPUT_BIND_PAUSE 0x06

#define INPUT_BIND_COUNT (INPUT_BIND_PAUSE + 1)

typedef struct {
  inputstate_t currentState[INPUT_BIND_COUNT];
  inputstate_t previousState[INPUT_BIND_COUNT];
} input_t;

extern input_t INPUT;

/**
 * Initializes the input system.
 */
void inputInit();

/**
 * Updates the input system.
 */
void inputUpdate();

/**
 * Returns whether or not the given input is down.
 * 
 * @param bind The input to check.
 */
inputstate_t inputIsDown(const inputbind_t bind);

/**
 * Returns whether or not the given input was pressed this update tick.
 * 
 * @param bind The input to check.
 */
inputstate_t inputWasPressed(const inputbind_t bind);

/**
 * Returns whether or not the given input was released this update tick.
 * 
 * @param bind The input to check.
 */
inputstate_t inputWasReleased(const inputbind_t bind);

/**
 * Returns the current state of the given input. Overridden per platform.
 * 
 * @param bind The input to check.
 */
inputstate_t inputGetState(const inputbind_t bind);