/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dawnopengl.h"
#include "input.h"
#include <GLFW/glfw3.h>

#define DAWN_GLFW_WIDTH_DEFAULT 640
#define DAWN_GLFW_HEIGHT_DEFAULT 480
#define DAWN_GLFW_BIND_KEYS_MAX 8

typedef struct {
  int32_t keys[DAWN_GLFW_BIND_KEYS_MAX];
  int32_t keyCount;
} dawnglfwbind_t;

typedef struct {
  GLFWwindow *window;
  int32_t width;
  int32_t height;
  dawnglfwbind_t inputBinds[INPUT_BIND_COUNT];
} dawnglfw_t;

extern dawnglfw_t DAWN_GLFW;

/**
 * Initialies GLFW.
 */
int32_t dawnGlfwStart();

/**
 * Binds a key to an input.
 * 
 * @param bind Bind to bind to.
 * @param key Key to bind.
 */
void dawnGlfwInputBind(inputbind_t bind, int32_t key);

/**
 * Event listener for when the window is resized.
 * 
 * @param window The window that was resized.
 * @param w The new width of the window.
 * @param h The new height of the window.
 */
void dawnGlfwOnResize(GLFWwindow *window, int32_t w, int32_t h);