/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "display/mesh.h"

#define PRIMITIVE_QUAD_VERTICE_COUNT 4
#define PRIMITIVE_QUAD_INDICE_COUNT 6

/**
 * Buffers a quad into the mesh.
 * 
 * @param mesh The mesh to buffer into.
 * @param xy0 The first xy coordinate.
 * @param xy1 The second xy coordinate.
 * @param uv0 The first uv coordinate.
 * @param uv1 The second uv coordinate.
 * @param color The color of the quad.
 * @param verticeIndex The vertice index to start at.
 * @param indiceIndex The indice index to start at.
 */
void primitiveQuadBuffer(
  mesh_t *mesh,
  const vec2_t xy0,
  const vec2_t xy1,
  const vec2_t uv0,
  const vec2_t uv1,
  const color4f_t color,
  const size_t verticeIndex,
  const size_t indiceIndex
);

/**
 * Buffers quad uv coordinates into the mesh.
 * 
 * @param mesh The mesh to buffer into.
 * @param uv0 The first uv coordinate.
 * @param uv1 The second uv coordinate.
 * @param verticeIndex The vertice index to start at.
 */
void primitiveQuadBufferCoordinates(
  mesh_t *mesh,
  const vec2_t uv0,
  const vec2_t uv1,
  int32_t verticeIndex
);