/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "display/texture.h"
#include "display/color.h"

typedef struct {
  texture_t texture;
  GLuint fboId;
  GLuint rboId;
} textureframebuffer_t;

/**
 * Initializes a texture framebuffer.
 * 
 * @param framebuffer Framebuffer to initialize.
 * @param width Width of the framebuffer (in pixels).
 * @param height Height of the framebuffer (in pixels).
 */
void textureFrameBufferInit(
  textureframebuffer_t *framebuffer,
  const int32_t width,
  const int32_t height
);

/**
 * Binds the framebuffer to the current context.
 * 
 * @param framebuffer Framebuffer to bind.
 */
void textureFrameBufferBind(const textureframebuffer_t *framebuffer);

/**
 * Clears the framebuffer.
 * 
 * @param framebuffer Framebuffer to clear.
 */
void textureFrameBufferClear(const textureframebuffer_t *framebuffer);

/**
 * Disposes of the framebuffer.
 * 
 * @param framebuffer Framebuffer to dispose.
 */
void textureFrameBufferDispose(textureframebuffer_t *framebuffer);